import { AssignmentStatus, User } from "next-auth";
import { useSession } from "next-auth/react";
import React, { createContext, useContext, useState, FC, ReactNode, useEffect } from "react";

interface GeneralContextProps {
  isSearchListOpen: boolean;
  currentUser: User;
  isAdmin: boolean;
  isClubOwner: boolean;
  isUser: boolean;
  batchStatuses: string[];
  fileStatus: string[];
  locationStatuses: string[];
  assignmentStatuses: AssignmentStatus[];
  newProjects: string[];
  toggleSearchList: (e: boolean) => void;
  saveProjectId: (e: string) => void;
  projectId: string;
}

const GeneralContext = createContext<GeneralContextProps | undefined>(undefined);

interface GeneralProviderProps {
  user: User;
  batchStatus: string[];
  locationStatus: string[];
  assignmentStatus: AssignmentStatus[];
  projects: string[];
  propIsAdmin?: boolean;
  propIsClubOwner?: boolean;
  propIsUser?: boolean;
  currentProjectId: string;
  children: ReactNode;
}


export const GeneralProvider: FC<GeneralProviderProps> = ({ user, batchStatus, locationStatus, assignmentStatus, projects, propIsAdmin, propIsClubOwner, propIsUser, currentProjectId, children }) => {
  const { data: session } = useSession();
  const [isSearchListOpen, setSearchListOpen] = useState<boolean>(false);
  const [projectId, setProjectId] = useState<string>(currentProjectId ?? "");
  const [isAdmin, setIsAdmin] = useState<boolean>(propIsAdmin ?? false);
  const [isClubOwner, setIsClubOwner] = useState<boolean>(propIsClubOwner ?? false);
  const [isUser, setIsUser] = useState<boolean>(propIsUser ?? false);
  const [currentUser, setCurrentUser] = useState<User>(user);
  const [batchStatuses, setBatchStatuses] = useState<string[]>(batchStatus ?? []);
  const [fileStatus, setFile] = useState<string[]>(['pending', 'in-process', 'completed', 'failed']);
  const [locationStatuses, setLocationStatuses] = useState<string[]>(locationStatus ?? []);
  const [assignmentStatuses, setAssignmentStatuses] = useState<AssignmentStatus[]>(assignmentStatus ?? []);
  const [newProjects, setNewProjects] = useState<string[]>(projects ?? []);

  const toggleSearchList = (e: boolean) => {
    setSearchListOpen((prev) => e);
  };

  const saveProjectId = (e: string) => {
    setProjectId(e);
  };

  const toggleIsAdmin = (e: boolean) => {
    setIsAdmin((prev) => e);
  };

  useEffect(() => {
    if (session?.user && !currentUser) {
      setCurrentUser(session.user);
    }
    if (session?.batchStatus && !batchStatus) {
      setBatchStatuses(session.batchStatus);
    }
    if (session?.locationStatus && !locationStatus) {
      setLocationStatuses(session.locationStatus);
    }
    if (session?.assignmentStatus && !assignmentStatus) {
      setAssignmentStatuses(session.assignmentStatus);
    }
    if (session?.projects && !newProjects) {
      setNewProjects(session.projects);
    }
  }, [session]);

  const contextValue: GeneralContextProps = {
    isSearchListOpen,
    currentUser,
    batchStatuses,
    fileStatus,
    locationStatuses,
    assignmentStatuses,
    newProjects,
    isAdmin,
    isClubOwner,
    isUser,
    toggleSearchList,
    saveProjectId,
    projectId,
  };

  return <GeneralContext.Provider value={contextValue}>{children}</GeneralContext.Provider>;
};

export const useGeneral = (): GeneralContextProps => {
  const context = useContext(GeneralContext);

  if (!context) {
    throw new Error("useGeneral must be used within a GeneralProvider");
  }

  return context;
};
